@@ -8,6 +8,7 @@ use crate::PythonInterpreter;
 use crate::Target;
 use anyhow::{bail, format_err, Context, Result};
 use cargo_metadata::{Metadata, MetadataCommand, Node};
+use regex::Regex;
 use serde::{Deserialize, Serialize};
 use std::collections::{HashMap, HashSet};
 use std::env;
@@ -99,10 +100,10 @@ impl Default for BuildOptions {
 impl BuildOptions {
     /// Tries to fill the missing metadata for a BuildContext by querying cargo and python
     pub fn into_build_context(self, release: bool, strip: bool) -> Result<BuildContext> {
-        let manifest_file = self
-            .manifest_path
-            .canonicalize()
-            .context(format_err!("Can't find {}", self.manifest_path.display()))?;
+        let manifest_file = &self.manifest_path;
+        if !manifest_file.exists() {
+            bail!("Can't find {}", self.manifest_path.display(),);
+        }
 
         if !manifest_file.is_file() {
             bail!(
@@ -110,15 +111,16 @@ impl BuildOptions {
                 self.manifest_path.display(),
                 manifest_file.display()
             );
-        };
+        }
 
         let cargo_toml = CargoToml::from_path(&manifest_file)?;
         let manifest_dir = manifest_file.parent().unwrap();
         let metadata21 = Metadata21::from_cargo_toml(&cargo_toml, &manifest_dir)
             .context("Failed to parse Cargo.toml into python metadata")?;
+        let extra_metadata = cargo_toml.remaining_core_metadata();
         let scripts = cargo_toml.scripts();
 
-        let crate_name = cargo_toml.package.name;
+        let crate_name = &cargo_toml.package.name;
 
         // If the package name contains minuses, you must declare a module with
         // underscores as lib name
@@ -128,10 +130,14 @@ impl BuildOptions {
             .and_then(|lib| lib.name.as_ref())
             .unwrap_or(&crate_name)
             .to_owned();
+        let extension_name = extra_metadata.name.as_ref().unwrap_or(&module_name);
 
-        let project_layout = ProjectLayout::determine(manifest_dir, &module_name)?;
+        let project_layout = ProjectLayout::determine(manifest_dir, &extension_name)?;
 
-        let cargo_extra_args = split_extra_args(&self.cargo_extra_args)?;
+        let mut cargo_extra_args = split_extra_args(&self.cargo_extra_args)?;
+        if let Some(ref target) = self.target {
+            cargo_extra_args.extend(vec!["--target".to_string(), target.clone()]);
+        }
 
         let cargo_metadata_extra_args = extract_cargo_metadata_args(&cargo_extra_args)?;
 
@@ -173,9 +179,9 @@ impl BuildOptions {
             // Only build a source distribution
             Some(ref interpreter) if interpreter.is_empty() => vec![],
             // User given list of interpreters
-            Some(interpreter) => find_interpreter(&bridge, &interpreter, &target)?,
+            Some(interpreter) => find_interpreter(&bridge, &interpreter, &target, None)?,
             // Auto-detect interpreters
-            None => find_interpreter(&bridge, &[], &target)?,
+            None => find_interpreter(&bridge, &[], &target, get_min_python_minor(&metadata21))?,
         };
 
         let rustc_extra_args = split_extra_args(&self.rustc_extra_args)?;
@@ -205,7 +211,7 @@ impl BuildOptions {
             project_layout,
             metadata21,
             scripts,
-            crate_name,
+            crate_name: crate_name.to_string(),
             module_name,
             manifest_path: self.manifest_path,
             out: wheel_dir,
@@ -222,6 +228,29 @@ impl BuildOptions {
     }
 }
 
+/// Uses very simple PEP 440 subset parsing to determine the
+/// minimum supported python minor version for interpreter search
+fn get_min_python_minor(metadata21: &Metadata21) -> Option<usize> {
+    if let Some(requires_python) = &metadata21.requires_python {
+        let regex = Regex::new(r#">=3\.(\d+)(?:\.\d)?"#).unwrap();
+        if let Some(captures) = regex.captures(&requires_python) {
+            let min_python_minor = captures[1]
+                .parse::<usize>()
+                .expect("Regex must only match usize");
+            Some(min_python_minor)
+        } else {
+            println!(
+                "⚠  Couldn't parse the value of requires-python, \
+                    not taking it into account when searching for python interpreter. \
+                    Note: Only `>=3.x.y` is currently supported."
+            );
+            None
+        }
+    } else {
+        None
+    }
+}
+
 /// pyo3 supports building abi3 wheels if the unstable-api feature is not selected
 fn has_abi3(cargo_metadata: &Metadata) -> Result<Option<(u8, u8)>> {
     let resolve = cargo_metadata
@@ -381,6 +410,7 @@ pub fn find_interpreter(
     bridge: &BridgeModel,
     interpreter: &[PathBuf],
     target: &Target,
+    min_python_minor: Option<usize>,
 ) -> Result<Vec<PythonInterpreter>> {
     match bridge {
         BridgeModel::Bindings(_) => {
@@ -388,7 +418,7 @@ pub fn find_interpreter(
                 PythonInterpreter::check_executables(&interpreter, &target, &bridge)
                     .context("The given list of python interpreters is invalid")?
             } else {
-                PythonInterpreter::find_all(&target, &bridge)
+                PythonInterpreter::find_all(&target, &bridge, min_python_minor)
                     .context("Finding python interpreters failed")?
             };
 
@@ -428,7 +458,7 @@ pub fn find_interpreter(
                         abiflags: "".to_string(),
                         target: target.clone(),
                         executable: PathBuf::new(),
-                        ext_suffix: Some(".pyd".to_string()),
+                        ext_suffix: ".pyd".to_string(),
                         interpreter_kind: InterpreterKind::CPython,
                         abi_tag: None,
                         libs_dir: PathBuf::from(manual_base_prefix),
@@ -657,4 +687,13 @@ mod test {
 
         assert_eq!(extract_cargo_metadata_args(&args).unwrap(), expected);
     }
+
+    #[test]
+    fn test_get_min_python_minor() {
+        // Nothing specified
+        let cargo_toml = CargoToml::from_path("test-crates/pyo3-pure/Cargo.toml").unwrap();
+        let metadata21 =
+            Metadata21::from_cargo_toml(&cargo_toml, &"test-crates/pyo3-pure").unwrap();
+        assert_eq!(get_min_python_minor(&metadata21), None);
+    }
 }
