@@ -1,12 +1,16 @@
-use crate::{Manylinux, PythonInterpreter};
-use anyhow::{bail, format_err, Result};
-use platform_info::*;
-use platforms::target::Env;
-use platforms::Platform;
 use std::env;
 use std::fmt;
 use std::path::Path;
 use std::path::PathBuf;
+use std::process::Command;
+use std::str;
+
+use anyhow::{bail, format_err, Context, Result};
+use platform_info::*;
+use platforms::target::Env;
+use platforms::Platform;
+
+use crate::{Manylinux, PythonInterpreter};
 
 /// All supported operating system
 #[derive(Debug, Clone, Eq, PartialEq)]
@@ -15,6 +19,19 @@ enum Os {
     Windows,
     Macos,
     FreeBsd,
+    OpenBsd,
+}
+
+impl fmt::Display for Os {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match *self {
+            Os::Linux => write!(f, "Linux"),
+            Os::Windows => write!(f, "Windows"),
+            Os::Macos => write!(f, "MacOS"),
+            Os::FreeBsd => write!(f, "FreeBSD"),
+            Os::OpenBsd => write!(f, "OpenBSD"),
+        }
+    }
 }
 
 /// All supported CPU architectures
@@ -26,6 +43,7 @@ pub enum Arch {
     Powerpc64,
     X86,
     X86_64,
+    S390X,
 }
 
 impl fmt::Display for Arch {
@@ -37,17 +55,35 @@ impl fmt::Display for Arch {
             Arch::Powerpc64 => write!(f, "ppc64"),
             Arch::X86 => write!(f, "i686"),
             Arch::X86_64 => write!(f, "x86_64"),
+            Arch::S390X => write!(f, "s390x"),
         }
     }
 }
 
+// Returns the set of supported architectures for each operating system
+fn get_supported_architectures(os: &Os) -> Vec<Arch> {
+    match os {
+        Os::Linux => vec![
+            Arch::Aarch64,
+            Arch::Armv7L,
+            Arch::Powerpc64,
+            Arch::Powerpc64Le,
+            Arch::X86,
+            Arch::X86_64,
+        ],
+        Os::Windows => vec![Arch::X86, Arch::X86_64],
+        Os::Macos => vec![Arch::Aarch64, Arch::X86_64],
+        Os::FreeBsd => vec![Arch::X86_64],
+        Os::OpenBsd => vec![Arch::X86_64],
+    }
+}
+
 /// The part of the current platform that is relevant when building wheels and is supported
 #[derive(Debug, Clone, Eq, PartialEq)]
 pub struct Target {
     os: Os,
     arch: Arch,
     env: Option<Env>,
-    triple: &'static str,
 }
 
 impl Target {
@@ -60,8 +96,9 @@ impl Target {
             Platform::find(target_triple)
                 .ok_or_else(|| format_err!("Unknown target triple {}", target_triple))?
         } else {
-            Platform::guess_current()
-                .ok_or_else(|| format_err!("Could guess the current platform"))?
+            let target_triple = get_host_target()?;
+            Platform::find(&target_triple)
+                .ok_or_else(|| format_err!("Unknown target triple {}", target_triple))?
         };
 
         let os = match platform.target_os {
@@ -69,6 +106,7 @@ impl Target {
             platforms::target::OS::Windows => Os::Windows,
             platforms::target::OS::MacOS => Os::Macos,
             platforms::target::OS::FreeBSD => Os::FreeBsd,
+            platforms::target::OS::OpenBSD => Os::OpenBsd,
             unsupported => bail!("The operating system {:?} is not supported", unsupported),
         };
 
@@ -87,34 +125,81 @@ impl Target {
             {
                 Arch::Powerpc64Le
             }
-            unsupported => bail!("The architecture {:?} is not supported", unsupported),
+            unsupported => bail!("The architecture {} is not supported", unsupported),
         };
 
-        // bail on any unsupported targets
-        match (&os, &arch) {
-            (Os::FreeBsd, Arch::Aarch64) => bail!("aarch64 is not supported for FreeBSD"),
-            (Os::FreeBsd, Arch::Armv7L) => bail!("armv7l is not supported for FreeBSD"),
-            (Os::FreeBsd, Arch::X86) => bail!("32-bit wheels are not supported for FreeBSD"),
-            (Os::FreeBsd, Arch::X86_64) => {
-                match PlatformInfo::new() {
-                    Ok(_) => {}
-                    Err(error) => bail!(error),
-                };
-            }
-            (Os::Macos, Arch::Armv7L) => bail!("armv7l is not supported for macOS"),
-            (Os::Macos, Arch::X86) => bail!("32-bit wheels are not supported for macOS"),
-            (Os::Windows, Arch::Aarch64) => bail!("aarch64 is not supported for Windows"),
-            (Os::Windows, Arch::Armv7L) => bail!("armv7l is not supported for Windows"),
-            (_, _) => {}
+        if !get_supported_architectures(&os).contains(&arch) {
+            bail!("{} is not supported on {}", arch, os);
         }
+
         Ok(Target {
             os,
             arch,
             env: platform.target_env,
-            triple: platform.target_triple,
         })
     }
 
+    /// Returns the platform part of the tag for the wheel name for cffi wheels
+    pub fn get_platform_tag(&self, manylinux: &Manylinux, universal2: bool) -> String {
+        match (&self.os, &self.arch) {
+            (Os::FreeBsd, Arch::X86_64) => {
+                let info = match PlatformInfo::new() {
+                    Ok(info) => info,
+                    Err(error) => panic!("{}", error),
+                };
+                let release = info.release().replace(".", "_").replace("-", "_");
+                format!("freebsd_{}_amd64", release)
+            }
+            (Os::OpenBsd, Arch::X86_64) => {
+                let info = match PlatformInfo::new() {
+                    Ok(info) => info,
+                    Err(error) => panic!("{}", error),
+                };
+                let release = info.release().replace(".", "_").replace("-", "_");
+                format!("openbsd_{}_amd64", release)
+            }
+            (Os::Linux, _) => format!("{}_{}", manylinux, self.arch),
+            (Os::Macos, Arch::X86_64) => {
+                if universal2 {
+                    "macosx_10_9_universal2".to_string()
+                } else {
+                    "macosx_10_7_x86_64".to_string()
+                }
+            }
+            (Os::Macos, Arch::Aarch64) => {
+                if universal2 {
+                    "macosx_10_9_universal2".to_string()
+                } else {
+                    "macosx_11_0_arm64".to_string()
+                }
+            }
+            (Os::Windows, Arch::X86) => "win32".to_string(),
+            (Os::Windows, Arch::X86_64) => "win_amd64".to_string(),
+            (_, _) => panic!("unsupported target should not have reached get_platform_tag()"),
+        }
+    }
+
+    /// Returns the name python uses in `sys.platform` for this os
+    pub fn get_python_os(&self) -> &str {
+        match self.os {
+            Os::Windows => "windows",
+            Os::Linux => "linux",
+            Os::Macos => "darwin",
+            Os::FreeBsd => "freebsd",
+            Os::OpenBsd => "openbsd",
+        }
+    }
+
+    /// Returns the default Manylinux tag for this architecture
+    pub fn get_default_manylinux_tag(&self) -> Manylinux {
+        match self.arch {
+            Arch::Aarch64 | Arch::Armv7L | Arch::Powerpc64 | Arch::Powerpc64Le | Arch::S390X => {
+                Manylinux::Manylinux2014
+            }
+            Arch::X86 | Arch::X86_64 => Manylinux::Manylinux2010,
+        }
+    }
+
     /// Returns whether the platform is 64 bit or 32 bit
     pub fn pointer_width(&self) -> usize {
         match self.arch {
@@ -124,6 +209,15 @@ impl Target {
             Arch::Powerpc64Le => 64,
             Arch::X86 => 32,
             Arch::X86_64 => 64,
+            Arch::S390X => 64,
+        }
+    }
+
+    /// Returns true if the current platform is not windows
+    pub fn is_unix(&self) -> bool {
+        match self.os {
+            Os::Windows => false,
+            Os::Linux | Os::Macos | Os::FreeBsd | Os::OpenBsd => true,
         }
     }
 
@@ -132,16 +226,6 @@ impl Target {
         self.arch
     }
 
-    /// Returns target triple string
-    pub fn target_triple(&self) -> &str {
-        &self.triple
-    }
-
-    /// Returns true if the current platform is linux or mac os
-    pub fn is_unix(&self) -> bool {
-        self.os != Os::Windows
-    }
-
     /// Returns true if the current platform is linux
     pub fn is_linux(&self) -> bool {
         self.os == Os::Linux
@@ -171,48 +255,6 @@ impl Target {
         }
     }
 
-    /// Returns the default Manylinux tag for this architecture
-    pub fn get_default_manylinux_tag(&self) -> Manylinux {
-        match self.arch {
-            Arch::Aarch64 | Arch::Armv7L | Arch::Powerpc64 | Arch::Powerpc64Le => {
-                Manylinux::Manylinux2014
-            }
-            Arch::X86 | Arch::X86_64 => Manylinux::Manylinux2010,
-        }
-    }
-
-    /// Returns the platform part of the tag for the wheel name for cffi wheels
-    pub fn get_platform_tag(&self, manylinux: &Manylinux, universal2: bool) -> String {
-        match (&self.os, &self.arch) {
-            (Os::FreeBsd, Arch::X86_64) => {
-                let info = match PlatformInfo::new() {
-                    Ok(info) => info,
-                    Err(error) => panic!("{}", error),
-                };
-                let release = info.release().replace(".", "_").replace("-", "_");
-                format!("freebsd_{}_amd64", release)
-            }
-            (Os::Linux, _) => format!("{}_{}", manylinux, self.arch),
-            (Os::Macos, Arch::X86_64) => {
-                if universal2 {
-                    "macosx_10_9_universal2".to_string()
-                } else {
-                    "macosx_10_7_x86_64".to_string()
-                }
-            }
-            (Os::Macos, Arch::Aarch64) => {
-                if universal2 {
-                    "macosx_10_9_universal2".to_string()
-                } else {
-                    "macosx_11_0_arm64".to_string()
-                }
-            }
-            (Os::Windows, Arch::X86) => "win32".to_string(),
-            (Os::Windows, Arch::X86_64) => "win_amd64".to_string(),
-            (_, _) => panic!("unsupported target should not have reached get_platform_tag()"),
-        }
-    }
-
     /// Returns the tags for the WHEEL file for cffi wheels
     pub fn get_py3_tags(&self, manylinux: &Manylinux, universal2: bool) -> Vec<String> {
         vec![format!(
@@ -221,29 +263,6 @@ impl Target {
         )]
     }
 
-    /// Returns the platform for the tag in the shared libraries file name
-    pub fn get_shared_platform_tag(&self) -> &'static str {
-        match (&self.os, &self.arch) {
-            (Os::FreeBsd, _) => "", // according imp.get_suffixes(), there are no such
-            (Os::Linux, Arch::Aarch64) => "aarch64-linux-gnu", // aka armv8-linux-gnueabihf
-            (Os::Linux, Arch::Armv7L) => "arm-linux-gnueabihf",
-            (Os::Linux, Arch::Powerpc64) => "powerpc64-linux-gnu",
-            (Os::Linux, Arch::Powerpc64Le) => "powerpc64le-linux-gnu",
-            (Os::Linux, Arch::X86) => "i386-linux-gnu", // not i686
-            (Os::Linux, Arch::X86_64) => "x86_64-linux-gnu",
-            (Os::Macos, Arch::X86_64) => "darwin",
-            (Os::Macos, Arch::Aarch64) => "darwin",
-            (Os::Windows, Arch::X86) => "win32",
-            (Os::Windows, Arch::X86_64) => "win_amd64",
-            (Os::Macos, _) => {
-                panic!("unsupported macOS Arch should not have reached get_shared_platform_tag()")
-            }
-            (Os::Windows, _) => {
-                panic!("unsupported Windows Arch should not have reached get_shared_platform_tag()")
-            }
-        }
-    }
-
     /// Returns the path to the python executable inside a venv
     pub fn get_venv_python(&self, venv_base: impl AsRef<Path>) -> PathBuf {
         if self.is_windows() {
@@ -316,3 +335,26 @@ impl Target {
         (tag, tags)
     }
 }
+
+fn get_host_target() -> Result<String> {
+    let output = Command::new("rustc")
+        .arg("-vV")
+        .output()
+        .context("Failed to run rustc to get the host target")?;
+    let output = str::from_utf8(&output.stdout).context("`rustc -vV` didn't return utf8 output")?;
+
+    let field = "host: ";
+    let host = output
+        .lines()
+        .find(|l| l.starts_with(field))
+        .map(|l| &l[field.len()..])
+        .ok_or_else(|| {
+            format_err!(
+                "`rustc -vV` didn't have a line for `{}`, got:\n{}",
+                field.trim(),
+                output
+            )
+        })?
+        .to_string();
+    Ok(host)
+}
